<?php
namespace codemania\core;
/**
 * CodeMania Content Manager Framework
 * 
 * @author CodeMania Team <team@codemania.ru>
 * @package CodeMania CMF
 * @version 1.0.0
 * @copyright (c) 2012, CodeMania Team
 * @link  http://codemania.ru
 * @license http://codemania.ru/license.html
 */
defined('CM_ROOT') or exit('Запрет доступа!');
/**
 * Роутер
 */
class Route extends Singleton {
    /**
     * Контроллер по-умолчанию
     * @var string
     */
    static protected $_default;
    /**
     * Правила роутинга
     * @var array
     */
    static private $_rules = array();
    /**
     * Массив шаблонов для замены
     * @var array 
     */
    static private  $_repl = array(
        array(':int', ':str'),
        array('[0-9]*', '.+'),
    );
    /**
     * Массив ключей => значений сегментов
     * @var array
     */
    static private $_routes = array();
    /**
     * Сохраняет контроллер по умолчанию
     * @param string $controller Имя контроллера
     */
    static public function set_controller($controller) {
        self::$_default = (string)ucfirst($controller);
    }
    /**
     * Сохраняет правила роутинга
     * @param array $rules Правила роутинга
     */
    static public function set_rules(array $rules) {
        self::$_rules = $rules;
    }
    /**
     * Возвращает значение сегмента по ключу
     * См. Route->_routes_combine();
     * @param string $name Ключ сегмента
     * @return string Значение сегмента по ключу 
     * или false в случаее его отсутствия
     */
    static public function get($name) {
        // Если ключ не существует возвращаем false
        return isset(self::$_routes[(string)$name]);
    }
    /**
     * Обработанная строка URI
     * @var string
     */
    private $_uri_string;
    /**
     * Массив сегментов
     * @var array
     */
    protected $_segments = array();
    /**
     * Определяет сегменты
     * @param string $uri_string Обработанная строка URI
     */
    public function __construct($uri_string) {
        // Записываем строку в обьект класса
        $this->_uri_string = (string)$uri_string;
        // Если строка не обработалась чесрез правила
        if ( ! $this->_execute_rules()) {
            // То просто разбиваем ее на сегменты
            $this->_fetch_segments();
        }
        // Комбинируем роуты
        $this->_routes_combine();
    }
    /**
     * @author ExpressionEngine Dev Team
     * @package CodeIgniter
     * Обрабатывает URI по правилу
     * @return boolean true в случае успешной обработки false в случае неудачи
     */
    private function _execute_rules() {
        // Если в массиве $_rules есть хоть одно правило 
        if (count(self::$_rules) > 0) {
            // 
            foreach (self::$_rules as $key => $val) {
                // Обрабатываем условное правило в Perl-совместимое
                $key = str_replace(self::$_repl[0], self::$_repl[1], $key);
                // Проверяем соответствие строки правилу
                if (preg_match('#^'.$key.'$#', $this->_uri_string)) {
                    // Если в строке не найдены элементы правил
                    if ((strpos($val, '$') && strpos($key, '(')) === false) {
                        // Возвращаем отрицательный результат
                        return false;
                    }
                    // Обрабатываем строку получившимся правилом
                    $val = preg_replace('#^'.$key.'$#', $val, $this->_uri_string); 
                    // Разбиваем строку на сегменты
                    $this->_segments = explode('/', $val);
                    // Возвращаем положительный результат
                    return true;
                }
            }
        }
        // Возвращаем отрицательный результат
        return false;
    }
    /**
     * Разбивает строку на сегменты
     */
    private function _fetch_segments() {
        $this->_segments = explode('/', $this->_uri_string);
    } 
    /**
     * Комбинирует из сегментов массив ключ => значение
     * Например из строки /edit/user/id/111/
     * функция вернет массив 
     * array(
     * 'edit' => 'user',
     * 'id' => '111'
     * )
     */
    private function _routes_combine() {
        // Ключи 
        $keys = array();
        // Значения
        $values = array();
        foreach ($this->_segments as $key => $val) {
            // Если ключ четный
            if ($key % 2) {
                // Записываем его в массив значений
                $values[] = $val;
            } else {
                // А если нет в массив ключей
                $keys[] = $val;
            }
        }
        // Если количество ключей равно количесву значений
        if (count($keys) === count($values)) {
            // Комбинируем единый массив
            self::$_routes = array_combine($keys, $values);
        }
    }
}